@@ -1,13 +1,17 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2015, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package acl
 
 import (
 	"fmt"
+	"os"
+	"regexp"
+	"strings"
 	"testing"
 
 	"github.com/hashicorp/nomad/ci"
+	"github.com/shoenig/test"
 	"github.com/shoenig/test/must"
 )
 
@@ -188,6 +192,8 @@ func TestParse(t *testing.T) {
 							NamespaceCapabilityReadScalingPolicy,
 							NamespaceCapabilityReadJobScaling,
 							NamespaceCapabilityScaleJob,
+							NamespaceCapabilityReadJob,
+							NamespaceCapabilitySubmitRecommendation,
 						},
 					},
 				},
@@ -412,6 +418,8 @@ func TestParse(t *testing.T) {
 							NamespaceCapabilityReadScalingPolicy,
 							NamespaceCapabilityReadJobScaling,
 							NamespaceCapabilityScaleJob,
+							NamespaceCapabilityReadJob,
+							NamespaceCapabilitySubmitRecommendation,
 						},
 					},
 				},
@@ -1099,3 +1107,74 @@ func TestPluginPolicy_isValid(t *testing.T) {
 		})
 	}
 }
+
+// TestPolicy_isCapabilityValid reads through the source to get all our
+// fine-grained capability constants and makes sure they're all marked as
+// valid. This prevents us from adding a new capability without marking it as
+// valid
+func TestPolicy_isCapabilityValid(t *testing.T) {
+	filename := "policy.go"
+	src, err := os.ReadFile(filename)
+	must.NoError(t, err)
+
+	// namespace
+	caps := []string{}
+	re := regexp.MustCompile(`(?m)^(?:\tNamespaceCapability[A-Za-z0-9]+\s+= ")(.*)"$`)
+	subs := re.FindAllSubmatch(src, -1)
+	for _, sub := range subs {
+		if len(sub) > 1 {
+			caps = append(caps, string(sub[1]))
+		}
+	}
+	must.Greater(t, 0, len(caps), must.Sprint("no constants match NamespaceCapability regex"))
+	for _, cap := range caps {
+		cap = strings.TrimSpace(cap)
+		test.True(t, isNamespaceCapabilityValid(cap), test.Sprintf("%q not valid", cap))
+	}
+
+	// variables
+	caps = []string{}
+	re = regexp.MustCompile(`(?m)^(?:\tVariablesCapability[A-Za-z0-9]+\s+= ")(.*)"$`)
+	subs = re.FindAllSubmatch(src, -1)
+	for _, sub := range subs {
+		if len(sub) > 1 {
+			caps = append(caps, string(sub[1]))
+		}
+	}
+	must.Greater(t, 0, len(caps), must.Sprint("no constants match VariablesCapability regex"))
+	for _, cap := range caps {
+		cap = strings.TrimSpace(cap)
+		test.True(t, isPathCapabilityValid(cap), test.Sprintf("%q not valid", cap))
+	}
+
+	// host volumes
+	caps = []string{}
+	re = regexp.MustCompile(`(?m)^(?:\tHostVolumeCapability[A-Za-z0-9]+\s+= ")(.*)"$`)
+	subs = re.FindAllSubmatch(src, -1)
+	for _, sub := range subs {
+		if len(sub) > 1 {
+			caps = append(caps, string(sub[1]))
+		}
+	}
+	must.Greater(t, 0, len(caps), must.Sprint("no constants match HostVolumeCapability regex"))
+	for _, cap := range caps {
+		cap = strings.TrimSpace(cap)
+		test.True(t, isHostVolumeCapabilityValid(cap), test.Sprintf("%q not valid", cap))
+	}
+
+	// node pools
+	caps = []string{}
+	re = regexp.MustCompile(`(?m)^(?:\tNodePoolCapability[A-Za-z0-9]+\s+= ")(.*)"$`)
+	subs = re.FindAllSubmatch(src, -1)
+	for _, sub := range subs {
+		if len(sub) > 1 {
+			caps = append(caps, string(sub[1]))
+		}
+	}
+	must.Greater(t, 0, len(caps), must.Sprint("no constants match NodePoolCapability regex"))
+	for _, cap := range caps {
+		cap = strings.TrimSpace(cap)
+		test.True(t, isNodePoolCapabilityValid(cap), test.Sprintf("%q not valid", cap))
+	}
+
+}
