@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2015, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package template
@@ -2571,7 +2571,7 @@ func Test_newRunnerConfig_consul(t *testing.T) {
 				ClientConfig: config.DefaultConfig(),
 			},
 			expectedOutputConfig: &ctconf.ConsulConfig{
-				Address:   pointer.Of("127.0.0.1:8500"),
+				Address:   pointer.Of("localhost:8500"),
 				Namespace: pointer.Of(""),
 				Auth:      ctconf.DefaultAuthConfig(),
 				Retry:     ctconf.DefaultRetryConfig(),
@@ -2588,7 +2588,7 @@ func Test_newRunnerConfig_consul(t *testing.T) {
 				ClientConfig: config.DefaultConfig(),
 			},
 			expectedOutputConfig: &ctconf.ConsulConfig{
-				Address:   pointer.Of("127.0.0.1:8500"),
+				Address:   pointer.Of("localhost:8500"),
 				Namespace: pointer.Of(""),
 				Auth:      ctconf.DefaultAuthConfig(),
 				Retry:     ctconf.DefaultRetryConfig(),
