@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2015, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package nomad
@@ -83,6 +83,8 @@ func TestPlanApply_applyPlan(t *testing.T) {
 	// Register alloc, deployment and deployment update
 	alloc := mock.Alloc()
 	must.NoError(t, s1.State().UpsertJobSummary(1000, mock.JobSummary(alloc.JobID)))
+	must.NoError(t, s1.State().UpsertJob(structs.MsgTypeTestSetup, 1001, nil, alloc.Job))
+
 	// Create an eval
 	eval := mock.Eval()
 	eval.JobID = alloc.JobID
@@ -102,7 +104,10 @@ func TestPlanApply_applyPlan(t *testing.T) {
 
 	// Create the plan with a deployment
 	plan := &structs.Plan{
-		Job:               alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		Deployment:        dnew,
 		DeploymentUpdates: updates,
 		EvalID:            eval.ID,
@@ -162,6 +167,7 @@ func TestPlanApply_applyPlan(t *testing.T) {
 	allocEvict.Job = nil
 	alloc2 := mock.Alloc()
 	must.NoError(t, s1.State().UpsertJobSummary(1500, mock.JobSummary(alloc2.JobID)))
+	must.NoError(t, s1.State().UpsertJob(structs.MsgTypeTestSetup, 1501, nil, alloc2.Job))
 	planRes = &structs.PlanResult{
 		NodeUpdate: map[string][]*structs.Allocation{
 			node.ID: {allocEvict},
@@ -177,7 +183,10 @@ func TestPlanApply_applyPlan(t *testing.T) {
 
 	// Apply the plan
 	plan = &structs.Plan{
-		Job:    job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: job.Namespace,
+			ID:        job.ID,
+		},
 		EvalID: eval.ID,
 	}
 	future, err = s1.applyPlan(plan, planRes, snap)
@@ -265,6 +274,9 @@ func TestPlanApply_applyPlanWithNormalizedAllocs(t *testing.T) {
 		PreemptedByAllocation: alloc.ID,
 	}
 	must.NoError(t, s1.State().UpsertJobSummary(1000, mock.JobSummary(alloc.JobID)))
+	must.NoError(t, s1.State().UpsertJob(structs.MsgTypeTestSetup, 1001, nil, alloc.Job))
+	must.NoError(t, s1.State().UpsertJob(structs.MsgTypeTestSetup, 1002, nil, stoppedAlloc.Job))
+	must.NoError(t, s1.State().UpsertJob(structs.MsgTypeTestSetup, 1003, nil, preemptedAlloc.Job))
 	must.NoError(t, s1.State().UpsertAllocs(structs.MsgTypeTestSetup, 1100, []*structs.Allocation{stoppedAlloc, preemptedAlloc}))
 
 	// Create an eval
@@ -295,11 +307,15 @@ func TestPlanApply_applyPlanWithNormalizedAllocs(t *testing.T) {
 
 	// Create the plan with a deployment
 	plan := &structs.Plan{
-		Job:               alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		Deployment:        dnew,
 		DeploymentUpdates: updates,
 		EvalID:            eval.ID,
 	}
+	must.NoError(t, s1.State().UpsertJob(structs.MsgTypeTestSetup, 1000, nil, alloc.Job))
 
 	// Apply the plan
 	future, err := s1.applyPlan(plan, planRes, snap)
@@ -463,13 +479,17 @@ func TestPlanApply_KeyringNotReady(t *testing.T) {
 		},
 	}
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
 		Deployment:        deploy,
 		DeploymentUpdates: dupdates,
 	}
+	must.NoError(t, srv.State().UpsertJob(structs.MsgTypeTestSetup, 1000, nil, alloc.Job))
 
 	planRes := &structs.PlanResult{
 		NodeAllocation: map[string][]*structs.Allocation{
@@ -495,7 +515,10 @@ func TestPlanApply_EvalPlan_Simple(t *testing.T) {
 
 	alloc := mock.Alloc()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
@@ -586,6 +609,7 @@ func TestPlanApply_EvalPlan_Preemption(t *testing.T) {
 	}
 
 	// Insert a preempted alloc such that the alloc will fit only after preemption
+	must.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, preemptedAlloc.Job))
 	state.UpsertAllocs(structs.MsgTypeTestSetup, 1001, []*structs.Allocation{preemptedAlloc})
 
 	alloc := mock.Alloc()
@@ -614,7 +638,11 @@ func TestPlanApply_EvalPlan_Preemption(t *testing.T) {
 		},
 	}
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
+		EvalID: uuid.Generate(),
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
@@ -666,7 +694,10 @@ func TestPlanApply_EvalPlan_Partial(t *testing.T) {
 	d.TaskGroups["web"].PlacedCanaries = []string{alloc.ID, alloc2.ID}
 
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID:  {alloc},
 			node2.ID: {alloc2},
@@ -719,7 +750,10 @@ func TestPlanApply_EvalPlan_Partial_AllAtOnce(t *testing.T) {
 	alloc2 := mock.Alloc() // Ensure alloc2 does not fit
 	alloc2.AllocatedResources = structs.NodeResourcesToAllocatedResources(node2.NodeResources)
 	plan := &structs.Plan{
-		Job:       alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		AllAtOnce: true, // Require all to make progress
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID:  {alloc},
@@ -766,7 +800,10 @@ func TestPlanApply_EvalNodePlan_Simple(t *testing.T) {
 
 	alloc := mock.Alloc()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
@@ -794,7 +831,10 @@ func TestPlanApply_EvalNodePlan_NodeNotReady(t *testing.T) {
 
 	alloc := mock.Alloc()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
@@ -821,7 +861,10 @@ func TestPlanApply_EvalNodePlan_NodeDrain(t *testing.T) {
 
 	alloc := mock.Alloc()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
@@ -847,7 +890,10 @@ func TestPlanApply_EvalNodePlan_NodeNotExist(t *testing.T) {
 	nodeID := "12345678-abcd-efab-cdef-123456789abc"
 	alloc := mock.Alloc()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			nodeID: {alloc},
 		},
@@ -875,7 +921,8 @@ func TestPlanApply_EvalNodePlan_NodeFull(t *testing.T) {
 	alloc.AllocatedResources = structs.NodeResourcesToAllocatedResources(node.NodeResources)
 	state.UpsertJobSummary(999, mock.JobSummary(alloc.JobID))
 	state.UpsertNode(structs.MsgTypeTestSetup, 1000, node)
-	state.UpsertAllocs(structs.MsgTypeTestSetup, 1001, []*structs.Allocation{alloc})
+	must.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 1001, nil, alloc.Job))
+	must.NoError(t, state.UpsertAllocs(structs.MsgTypeTestSetup, 1002, []*structs.Allocation{alloc}))
 
 	alloc2 := mock.Alloc()
 	alloc2.NodeID = node.ID
@@ -883,7 +930,10 @@ func TestPlanApply_EvalNodePlan_NodeFull(t *testing.T) {
 
 	snap, _ := state.Snapshot()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc2},
 		},
@@ -923,9 +973,10 @@ func TestPlanApply_EvalNodePlan_NodeFull_Device(t *testing.T) {
 		},
 	}
 
-	state.UpsertJobSummary(999, mock.JobSummary(alloc.JobID))
-	state.UpsertNode(structs.MsgTypeTestSetup, 1000, node)
-	state.UpsertAllocs(structs.MsgTypeTestSetup, 1001, []*structs.Allocation{alloc})
+	must.NoError(t, state.UpsertJobSummary(998, mock.JobSummary(alloc.JobID)))
+	must.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 999, nil, alloc.Job))
+	must.NoError(t, state.UpsertNode(structs.MsgTypeTestSetup, 1000, node))
+	must.NoError(t, state.UpsertAllocs(structs.MsgTypeTestSetup, 1001, []*structs.Allocation{alloc}))
 
 	// Alloc2 tries to use the same device
 	alloc2 := mock.Alloc()
@@ -943,7 +994,10 @@ func TestPlanApply_EvalNodePlan_NodeFull_Device(t *testing.T) {
 
 	snap, _ := state.Snapshot()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc2},
 		},
@@ -968,7 +1022,10 @@ func TestPlanApply_EvalNodePlan_UpdateExisting(t *testing.T) {
 	snap, _ := state.Snapshot()
 
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
@@ -995,12 +1052,16 @@ func TestPlanApply_EvalNodePlan_UpdateExisting_Ineligible(t *testing.T) {
 	node.SchedulingEligibility = structs.NodeSchedulingIneligible
 	alloc.NodeID = node.ID
 	alloc.AllocatedResources = structs.NodeResourcesToAllocatedResources(node.NodeResources)
-	state.UpsertNode(structs.MsgTypeTestSetup, 1000, node)
-	state.UpsertAllocs(structs.MsgTypeTestSetup, 1001, []*structs.Allocation{alloc})
+	must.NoError(t, state.UpsertNode(structs.MsgTypeTestSetup, 99, node))
+	must.NoError(t, state.UpsertJob(structs.MsgTypeTestSetup, 1000, nil, alloc.Job))
+	must.NoError(t, state.UpsertAllocs(structs.MsgTypeTestSetup, 1001, []*structs.Allocation{alloc}))
 	snap, _ := state.Snapshot()
 
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc},
 		},
@@ -1035,7 +1096,10 @@ func TestPlanApply_EvalNodePlan_NodeFull_Evict(t *testing.T) {
 	allocEvict.DesiredStatus = structs.AllocDesiredStatusEvict
 	alloc2 := mock.Alloc()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeUpdate: map[string][]*structs.Allocation{
 			node.ID: {allocEvict},
 		},
@@ -1071,7 +1135,10 @@ func TestPlanApply_EvalNodePlan_NodeFull_AllocEvict(t *testing.T) {
 
 	alloc2 := mock.Alloc()
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeAllocation: map[string][]*structs.Allocation{
 			node.ID: {alloc2},
 		},
@@ -1106,7 +1173,10 @@ func TestPlanApply_EvalNodePlan_NodeDown_EvictOnly(t *testing.T) {
 	*allocEvict = *alloc
 	allocEvict.DesiredStatus = structs.AllocDesiredStatusEvict
 	plan := &structs.Plan{
-		Job: alloc.Job,
+		JobInfo: &structs.PlanJobTuple{
+			Namespace: alloc.Job.Namespace,
+			ID:        alloc.Job.ID,
+		},
 		NodeUpdate: map[string][]*structs.Allocation{
 			node.ID: {allocEvict},
 		},
@@ -1188,7 +1258,10 @@ func TestPlanApply_EvalNodePlan_Node_Disconnected(t *testing.T) {
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
 			plan := &structs.Plan{
-				Job:            job,
+				JobInfo: &structs.PlanJobTuple{
+					Namespace: job.Namespace,
+					ID:        job.ID,
+				},
 				NodeAllocation: tc.nodeAllocs,
 			}
 
