@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2015, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package nomad
@@ -460,7 +460,8 @@ func TestSearch_PrefixSearch_Allocation(t *testing.T) {
 	summary := mock.JobSummary(alloc.JobID)
 	fsmState := s.fsm.State()
 
-	require.NoError(t, fsmState.UpsertJobSummary(999, summary))
+	must.NoError(t, fsmState.UpsertJobSummary(998, summary))
+	must.NoError(t, fsmState.UpsertJob(structs.MsgTypeTestSetup, 999, nil, alloc.Job))
 	require.NoError(t, fsmState.UpsertAllocs(structs.MsgTypeTestSetup, 90, []*structs.Allocation{alloc}))
 
 	prefix := alloc.ID[:len(alloc.ID)-2]
@@ -497,7 +498,8 @@ func TestSearch_PrefixSearch_All_UUID(t *testing.T) {
 	summary := mock.JobSummary(alloc.JobID)
 	fsmState := s.fsm.State()
 
-	require.NoError(t, fsmState.UpsertJobSummary(999, summary))
+	must.NoError(t, fsmState.UpsertJobSummary(998, summary))
+	must.NoError(t, fsmState.UpsertJob(structs.MsgTypeTestSetup, 999, nil, alloc.Job))
 	require.NoError(t, fsmState.UpsertAllocs(structs.MsgTypeTestSetup, 1000, []*structs.Allocation{alloc}))
 
 	node := mock.Node()
@@ -1627,7 +1629,8 @@ func TestSearch_FuzzySearch_Allocation(t *testing.T) {
 	summary := mock.JobSummary(alloc.JobID)
 	fsmState := s.fsm.State()
 
-	require.NoError(t, fsmState.UpsertJobSummary(999, summary))
+	must.NoError(t, fsmState.UpsertJobSummary(998, summary))
+	must.NoError(t, fsmState.UpsertJob(structs.MsgTypeTestSetup, 999, nil, alloc.Job))
 	require.NoError(t, fsmState.UpsertAllocs(structs.MsgTypeTestSetup, 90, []*structs.Allocation{alloc}))
 
 	req := &structs.FuzzySearchRequest{
