@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2015, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package structs
@@ -1093,20 +1093,28 @@ func TestTaskCSIPluginConfig_Equal(t *testing.T) {
 	}})
 }
 
-func TestCSISecretsSanitize(t *testing.T) {
+func TestCSIVolumeSanitize(t *testing.T) {
 	ci.Parallel(t)
 
-	orig := &CSISecrets{
-		"foo": "bar",
-		"baz": "qux",
+	orig := &CSIVolume{
+		ID: "foo",
+		MountOptions: &CSIMountOptions{
+			FSType:     "ext4",
+			MountFlags: []string{"ro", "noatime"},
+		},
+		Secrets: CSISecrets{
+			"foo": "bar",
+			"baz": "qux",
+		},
+		Parameters: map[string]string{"example": "unchanged"},
 	}
 
 	sanitized := orig.Sanitize()
-	must.NotEq(t, orig, sanitized)
+	must.Eq(t, []string{"[REDACTED]"}, sanitized.MountOptions.MountFlags)
+	must.Nil(t, sanitized.Secrets)
 
-	for _, v := range *sanitized {
-		must.Eq(t, v, "[REDACTED]")
-	}
+	orig.Parameters["example"] = "different"
+	must.Eq(t, "unchanged", sanitized.Parameters["example"])
 }
 
 func TestCSIMountOptionsSanitize(t *testing.T) {
