@@ -1,5 +1,5 @@
 /**
- * Copyright (c) HashiCorp, Inc.
+ * Copyright IBM Corp. 2015, 2025
  * SPDX-License-Identifier: BUSL-1.1
  */
 
@@ -13,18 +13,19 @@ export default class VersionTagAdapter extends ApplicationAdapter {
   urlForCreateRecord(_modelName, model) {
     const tagName = model.attr('name');
     const jobName = model.attr('jobName');
-    return `${this.buildURL()}/job/${jobName}/versions/${tagName}/tag`;
+    const namespace = model.attr('jobNamespace');
+    return `${this.buildURL()}/job/${jobName}/versions/${tagName}/tag?namespace=${namespace}`;
   }
 
-  async deleteTag(jobName, tagName) {
+  async deleteTag(namespace, jobName, tagName) {
     let deletion = this.ajax(
-      this.urlForDeleteRecord(jobName, tagName),
+      this.urlForDeleteRecord(namespace, jobName, tagName),
       'DELETE'
     );
     return deletion;
   }
 
-  urlForDeleteRecord(jobName, tagName) {
-    return `${this.buildURL()}/job/${jobName}/versions/${tagName}/tag`;
+  urlForDeleteRecord(namespace, jobName, tagName) {
+    return `${this.buildURL()}/job/${jobName}/versions/${tagName}/tag?namespace=${namespace}`;
   }
 }
